/*
Indicator:
	an abstract superclass (I had to have at least ONE in here, after all)
	of Ruler and Magnifier, which display some sort of graduated
	scale in a fixed rectangle of the screen.  The scale has a little
	"indicator" on it, to display the current value of some parameter.
	Subclasses must override drawPriv, getIndicatorPos, and setIndicatorPos.

Ruler: 
	Displays a graduated ruler, centered on zero (...-2cm,-1cm,0cm,1cm,2cm...)

Magnifier:
	Displays a magnification selector (0x,1x,2x,...)
*/

import java.awt.*;
/*Indicator should probably be a subclass of something in 
 java.awt... but I don't know what.*/
abstract public class Indicator
{
	double value;//The value of the indicator.
	int centerY;
	Rectangle bounds=new Rectangle();
	boolean eraseBackground=true;
//Initialize the indicator
	public void init(boolean erase,double Nvalue)
	{
		eraseBackground=erase;
		value=Nvalue;
	}
//Set up the bounds
	public void setSize(int startX,int endX, int startY, int endY)
	{
		bounds.x=startX;
		bounds.width=endX-startX;
		bounds.y=startY;
		bounds.height=endY-startY;
		centerY=bounds.y+bounds.height/2;
	}
	
//The scale reflects the distance in pixels between sucessive markings.
	double scale;
	public void setScale(double Nscale) 
	{
		scale=Nscale;
	}
	
	public double getValue() {return value;}
	public void setValue(double v) {value=v;}
	
//Draw the entire indicator to the specified Graphics.
	public void draw(Graphics g)
	{
		//Erase the background if needed
		g.setColor(Color.black);
		if (eraseBackground)
			g.fillRect(bounds.x,bounds.y,bounds.width,bounds.height);
		//Set up a new graphics clipped to the indicator's bounds
		Graphics newGraf=g.create(0,0,bounds.x+bounds.width,bounds.y+bounds.height);
		newGraf.clipRect(bounds.x,bounds.y,bounds.width,bounds.height);
		//Call descendants' draw routine
		drawPriv(newGraf);
		//Draw the indicator
		if (eraseBackground)
			drawIndicator(newGraf,bounds.x,getIndicatorPos());
	}
	abstract void drawPriv(Graphics g);
	abstract int getIndicatorPos();
//External entry to set the position of the indicator (e.g. on mouse down)
	public abstract void setIndicatorPos(int y);
//private routine: display a little white triangle at x,y
	void drawIndicator(Graphics g,int x,int y)
	{
		g.setColor(Color.white);
		int xPoints[]={x+5,x+20,x+5},
			yPoints[]={y-5,y,y+5};
		g.fillPolygon(xPoints,yPoints,3);
	}
};

