
/*Rendering Utilities:
This file contains a variety of small, focused classes 
largely devoted to software 3D rendering.

Line3D is a line segment through 3D space, which can be intersected
with the slicing plane (at z=0) to return a point.

QuadFace is a quadrilateral face, which can also be
intersected with z=0 to yeild an intersecting line segment (Line3D).

Span is an interval on the integers.  In C-style arrays,
"min" is included; "max" is not included.

Rastergon is a rasterized polygon.  This is just a span for each y-line
of the screen.  The polygon gets defined over several calls by passing 
different line segments to Rastergon.addLine.
*/
/*A line through three dimentional space*/
public class Line3D{
	public Vector pos,dir;/*Position and direction vector for line.*/
	public Line3D(Vector p1,Vector p2)/*p1,p2:Endpoints of line. Line goes from p1 to p2.*/
	{
		pos=p1;
		dir=p2.minus(p1);
	}
	/*Return if, and where, this line intersects the plane z=0.*/
	public boolean intersect(Vector where)
	{
		double t;
		if (dir.z==0.0)
			dir.z=0.0000001;
		t=-pos.z/dir.z;
		where.copyFrom(pos.plus(dir.scaleBy(t)));
		if (t>=0&&t<=1.0)
			return true;
		else return false;
	}
}
