/* Java Source File 
a part of SliceViewer
  a program written in 2/98 by Orion Lawlor.
Public domain source code.

Send questions to fsosl@uaf.edu
*/
/*Vector is really quite a simple class-- it's just
three cartesian coordinates.  These can be interpreted
as a position or a direction, depending on the
circumstance.

There are public routines to add, subtract, scale, 
take the dot and cross product, normalize, and
return the magnitude of vectors.

All the data members (all 3 of them) are public.
This is so routines like Matrix3D can directly
(i.e. efficiently) read and write vectors.
Many OOP-ers disagree with this-- as I would for
large or complex classes-- but for tiny classes
I see no reason not to allow direct acess to 
data members.  In Java, a language without structures,
this is doubly true.
*/

class Vector {
	public double x,y,z;
	public Vector(double X,double Y,double Z) {x=X;y=Y;z=Z;}
	public void copyFrom(Vector v)
		{x=v.x;y=v.y;z=v.z;}
	public double magnitude()
	{
		return Math.sqrt(x*x+y*y+z*z);
	}
	public Vector normalize(){return scaleBy(1.0/magnitude());}
	public Vector plus(Vector a)
		{return new Vector(x+a.x,y+a.y,z+a.z);}
	public Vector minus(Vector a)
		{return new Vector(x-a.x,y-a.y,z-a.z);}
	public Vector scaleBy(double scale)
		{return new Vector(scale*x,scale*y,scale*z);}
	public double dot(Vector a)
		{return x*a.x+y*a.y+z*a.z;}
	public Vector cross(Vector b)
	{/*Return right-handed cross product of this Vector and b*/
		return new Vector(y*b.z-z*b.y,
					  z*b.x-x*b.z,
					  x*b.y-y*b.x);
	}
};
